import { SchemaFromBackend } from '@graphpolaris/models';
import { SchemaUtils } from '@graphpolaris/schema-utils';
import { MultiGraph } from 'graphology';
import { Attributes } from 'graphology-types';
import {
  movieSchemaRaw,
  northwindSchemaRaw,
  simpleSchemaRaw,
  twitterSchemaRaw,
} from 'libs/shared/mock-data/src';

describe('SchemaUsecases', () => {
  test.each([
    { data: simpleSchemaRaw },
    { data: movieSchemaRaw },
    { data: northwindSchemaRaw },
    { data: twitterSchemaRaw },
  ])('parseSchemaFromBackend parsing should work', ({ data }) => {
    // console.log('testinput', input);

    const parsed = SchemaUtils.ParseSchemaFromBackend(
      data as SchemaFromBackend
    );
    expect(parsed).toBeDefined();

    let parsedNodeAttributes: Attributes[] = [];
    parsed.forEachNode((node, attr) => {
      // console.log('Node', node, attr);
      parsedNodeAttributes.push(attr.attributes);
    });

    let parsedEdgeAttributes: Attributes = [];
    parsed.forEachEdge((edge, attr, source, target, sa, ta, undirected) => {
      // console.log('Edge', edge, attr, source, target, sa, ta, undirected);
      parsedEdgeAttributes.push(attr.attribute);
    });

    expect(data.nodes.length).toEqual(parsed.order);
    expect(data.edges.length).toEqual(parsed.size);

    let inputNodeAttributes: Attributes = [];
    data.nodes.forEach((node) => {
      inputNodeAttributes.push(node.attributes as Attributes);
    });

    let inputEdgeAttributes: Attributes = [];
    data.edges.forEach((edge) => {
      inputEdgeAttributes.push(edge.attributes as Attributes);
    });

    expect(inputNodeAttributes).toEqual(parsedNodeAttributes);
    expect(inputEdgeAttributes).toEqual(parsedEdgeAttributes);
  });

  it('should export and reimport', () => {
    const parsed = SchemaUtils.ParseSchemaFromBackend(
      simpleSchemaRaw as SchemaFromBackend
    );
    const reload = MultiGraph.from(parsed.export());

    expect(parsed).toStrictEqual(reload);
  });

  test.each([
    { data: simpleSchemaRaw },
    { data: movieSchemaRaw },
    { data: northwindSchemaRaw },
    { data: twitterSchemaRaw },
  ])('should load my test json $data', ({ data }) => {
    expect(data).toBeDefined();
    expect(data.nodes).toBeDefined();
  });
});
