import Graph, { MultiGraph } from 'graphology';
// import cytoscape from 'cytoscape'; // eslint-disable-line
import { setSchema, store } from '@graphpolaris/shared/data-access/store';
import { Elements, Node, Edge } from 'react-flow-renderer';
import { SchemaFromBackend } from '@graphpolaris/models';
import { Attributes } from 'graphology-types';

type CytoNode = {
  data: {
    id: string;
    type: string;
    source?: string;
    target?: string;
    position?: {
      x: number;
      y: number;
    };
  };
};

// // Layouts a given schema
// export function handleSchemaLayout(graph: Graph): void {
//   const layout = createSchemaLayout(graph);

//   layout.then((cy) => {
//     //cy.cy.elements().forEach((elem) => {
//     cy.cy.nodes().forEach((elem: any) => {
//       const position = elem.position();
//       console.log(elem.id());

//       graph.setNodeAttribute(elem.id(), 'x', position.x);
//       graph.setNodeAttribute(elem.id(), 'y', position.y);
//     });

//     store.dispatch(setSchema(graph));
//   });
// }

// // Creates a schema layout (async)
// function createSchemaLayout(graph: Graph): Promise<cytoscape.EventObject> {
//   const cytonodes: CytoNode[] = trimSchema(graph);

//   const cy = cytoscape({
//     elements: cytonodes,
//   });

//   const options = {
//     name: 'cose',

//     // Whether to animate while running the layout
//     // true : Animate continuously as the layout is running
//     // false : Just show the end result
//     // 'end' : Animate with the end result, from the initial positions to the end positions
//     animate: true,

//     // Easing of the animation for animate:'end'
//     animationEasing: undefined,

//     // The duration of the animation for animate:'end'
//     animationDuration: undefined,

//     // A function that determines whether the node should be animated
//     // All nodes animated by default on animate enabled
//     // Non-animated nodes are positioned immediately when the layout starts
//     // animateFilter: function (node: any, i: any) {
//     //   return true;
//     // },

//     // The layout animates only after this many milliseconds for animate:true
//     // (prevents flashing on fast runs)
//     animationThreshold: 250,

//     // Number of iterations between consecutive screen positions update
//     refresh: 20,

//     // Whether to fit the network view after when done
//     fit: true,

//     // Padding on fit
//     padding: 30,

//     // Constrain layout bounds; { x1, y1, x2, y2 } or { x1, y1, w, h }
//     boundingBox: undefined,

//     // Excludes the label when calculating node bounding boxes for the layout algorithm
//     nodeDimensionsIncludeLabels: false,

//     // Randomize the initial positions of the nodes (true) or use existing positions (false)
//     randomize: false,

//     // Extra spacing between components in non-compound graphs
//     componentSpacing: 200, // 40

//     // Node repulsion (non overlapping) multiplier
//     nodeRepulsion: function (node: any) {
//       return 2048;
//     },

//     // Node repulsion (overlapping) multiplier
//     nodeOverlap: 4,

//     // Ideal edge (non nested) length
//     idealEdgeLength: function (edge: any) {
//       return 32;
//     },

//     // Divisor to compute edge forces
//     edgeElasticity: function (edge: any) {
//       return 32;
//     },

//     // Nesting factor (multiplier) to compute ideal edge length for nested edges
//     nestingFactor: 1.2,

//     // Gravity force (constant)
//     gravity: 1,

//     // Maximum number of iterations to perform
//     numIter: 1000,

//     // Initial temperature (maximum node displacement)
//     initialTemp: 1000,

//     // Cooling factor (how the temperature is reduced between consecutive iterations
//     coolingFactor: 0.99,

//     // Lower temperature threshold (below this point the layout will end)
//     minTemp: 1.0,
//   };

//   const layout = cy.layout(options);

//   layout.run();

//   return layout.pon('layoutstop');
// }

// Takes the schema as input and creates a list of nodes and edges in a format that the layouting algorithm can use.
function trimSchema(graph: Graph): CytoNode[] {
  const cytonodes: CytoNode[] = [];

  graph.forEachNode((node) => {
    cytonodes.push({
      data: { id: node, type: 'node' },
    });
  });

  graph.forEachEdge((edge, _attributes, source, target) => {
    cytonodes.push({
      data: { id: edge, type: 'edge', source: source, target: target },
    });
  });

  return cytonodes;
}

// Takes the schema as an imput and creates basic react flow elements for them.
export function createReactFlowElements(graph: Graph): Elements<Node | Edge> {
  const initialElements: Elements<Node | Edge> = [];

  graph.forEachNode((node: string, attributes: Attributes): void => {
    const newNode: Node = {
      id: node,
      data: {
        label: attributes.name,
      },
      position: { x: attributes.x, y: attributes.y },
    };
    initialElements.push(newNode);
  });

  graph.forEachEdge((edge, _attributes, source, target): void => {
    const newEdge: Edge = {
      id: edge,
      source: source,
      target: target,
    };
    initialElements.push(newEdge);
  });

  return initialElements;
}

// export function parseSchemaFromBackend(
//   schemaFromBackend: SchemaFromBackend
// ): Graph {
//   const { nodes, edges } = schemaFromBackend;
//   // Instantiate a directed graph that allows self loops and parallel edges
//   const schemaGraph = new MultiGraph({ allowSelfLoops: true });
//   // console.log('parsing schema');
//   // The graph schema needs a node for each node AND edge. These need then be connected

//   nodes.forEach((node) => {
//     schemaGraph.addNode(node.name, {
//       name: node.name,
//       attributes: node.attributes,
//       x: 0,
//       y: 0,
//     });
//   });

//   // The name of the edge will be name + from + to, since edge names are not unique
//   edges.forEach((edge) => {
//     const edgeID = [edge.name, '_', edge.from, edge.to].join(''); //ensure that all interpreted as string

//     // This node is the actual edge
//     schemaGraph.addDirectedEdgeWithKey(edgeID, edge.from, edge.to, {
//       attribute: edge.attributes,
//     });
//   });
//   return schemaGraph;
// }
