// import {
//   movieSchemaRaw,
//   northwindSchemaRaw,
//   simpleSchemaRaw,
//   twitterSchemaRaw,
// } from '@graphpolaris/shared/mock-data';
import Graph, { MultiGraph } from 'graphology';

import connectedCaveman from 'graphology-generators/community/connected-caveman';
import ladder from 'graphology-generators/classic/ladder';
import { LayoutFactory } from './layout-creator-usecase';


const TIMEOUT = 10;

/**
 * @jest-environment jsdom
 */
describe('LayoutFactory', () => {
  /**
   * @jest-environment jsdom
   */
  it('should work with noverlap from graphology ', () => {
    const graph = new MultiGraph();

    // Adding some nodes
    // graph.addNode('John', { x: 0, y: 0, width: 200, height: 200 });
    // graph.addNode('Martha', { x: 0, y: 0 });
    graph.addNode('John');
    graph.addNode('Martha');

    // Adding an edge
    graph.addEdge('John', 'Martha');

    const layoutFactory = new LayoutFactory();
    const layoutAlgorithm = layoutFactory.createLayout(
      'Graphology_noverlap'
    );
    layoutAlgorithm?.layout(graph);

    // const positionMap = new Set<string>();
    graph.forEachNode((node, attr) => {
      expect(graph.getNodeAttribute(node, 'x')).toBeDefined();
      expect(graph.getNodeAttribute(node, 'y')).toBeDefined();
      
      // const pos = '' + attr['x'] + '' + attr['y'];
      // expect(positionMap.has(pos)).toBeFalsy();
      // positionMap.add(pos);
    });
  }, TIMEOUT);

  test('should work with noverlap from graphology on generated graph', () => {
    // Creating a ladder graph
    const graph = ladder(Graph, 10);

    graph.forEachNode((node, attr) => {
      graph.setNodeAttribute(node, 'x', 0);
      graph.setNodeAttribute(node, 'y', 0);
    });

    const layoutFactory = new LayoutFactory();

    const layout = layoutFactory.createLayout('Graphology_noverlap');
    layout?.layout(graph);

    const positionMap = new Set<string>();
    graph.forEachNode((node, attr) => {
      const pos = '' + attr['x'] + '' + attr['y'];

      expect(positionMap.has(pos)).toBeFalsy();
      positionMap.add(pos);
    });
  }, TIMEOUT);

  test('should work with random from graphology on generated graph', () => {
    // Creating a ladder graph
    const graph = ladder(Graph, 10);

    graph.forEachNode((node, attr) => {
      graph.setNodeAttribute(node, 'x', 0);
      graph.setNodeAttribute(node, 'y', 0);
    });

    const layoutFactory = new LayoutFactory();

    const layout = layoutFactory.createLayout('Graphology_random');
    layout?.setDimensions(100, 100);
    layout?.layout(graph);

    const positionMap = new Set<string>();
    graph.forEachNode((node, attr) => {
      const pos = '' + attr['x'] + '' + attr['y'];

      expect(positionMap.has(pos)).toBeFalsy();
      positionMap.add(pos);
    });
  }, TIMEOUT);

  test('should work with circular from graphology on generated graph', () => {
    // Creating a ladder graph
    const graph = ladder(Graph, 100);

    graph.forEachNode((node) => {
      graph.setNodeAttribute(node, 'x', 0);
      graph.setNodeAttribute(node, 'y', 0);
    });

    const layoutFactory = new LayoutFactory();

    const layout = layoutFactory.createLayout('Graphology_circular');
    layout?.setDimensions(100, 100);
    layout?.layout(graph);

    const positionMap = new Set<string>();
    graph.forEachNode((node, attr) => {
      const pos = '' + attr['x'] + '' + attr['y'];

      expect(positionMap.has(pos)).toBeFalsy();
      positionMap.add(pos);
    });
  }, TIMEOUT);

  test('should work with Graphology_forceAtlas2 from graphology on generated graph', () => {
    // console.log(Object.keys(AllLayoutAlgorithms))

    const graph = connectedCaveman(Graph, 6, 8);

    graph.forEachNode((node, attr) => {
      expect(graph.getNodeAttribute(node, 'x')).toBeUndefined();
      expect(graph.getNodeAttribute(node, 'y')).toBeUndefined();
    });

    // console.log('before');
    const layoutFactory = new LayoutFactory();
    const layout = layoutFactory.createLayout('Graphology_forceAtlas2');
    layout?.setDimensions(100, 100);
    layout?.layout(graph);

    // console.log('after');

    const positionMap = new Set<string>();
    graph.forEachNode((node, attr) => {
      expect(graph.getNodeAttribute(node, 'x')).toBeDefined();
      expect(graph.getNodeAttribute(node, 'y')).toBeDefined();

      const pos = '' + attr['x'] + '' + attr['y'];
      console.log(pos);
      
      expect(positionMap.has(pos)).toBeFalsy();
      positionMap.add(pos);

      expect(isNaN(graph.getNodeAttribute(node, 'x'))).toBeFalsy();
      expect(isNaN(graph.getNodeAttribute(node, 'y'))).toBeFalsy();
    });
  }, TIMEOUT);
});
